Package: sdcTable
Version: 0.34.0
Date: 2026-05-09
Title: Methods for Statistical Disclosure Control in Tabular Data
Description: Methods for statistical disclosure control in
    tabular data such as primary and secondary cell suppression as described for example
    in Hundepol et al. (2012) <doi:10.1002/9781118348239> are covered in this package.
Authors@R: person("Bernhard", "Meindl", email = "bernhard.meindl@gmail.com", role = c("aut", "cre"))
URL: https://github.com/sdcTools/sdcTable
BugReports: https://github.com/sdcTools/userSupport/issues
Depends: R (>= 3.5.0), Rcpp (>= 0.11.0), sdcHierarchies (>= 0.19.1)
Imports: data.table, knitr, rlang, stringr, methods, slam, progressr,
        utils, Matrix (>= 1.3-0), SSBtools, highs
Suggests: testthat (>= 0.3), rmarkdown, webshot, RegSDC, future.apply,
        future
Config/testthat/edition: 3
LinkingTo: Rcpp
License: GPL (>= 2)
LazyData: true
SystemRequirements: GLPK library, including -dev or -devel part
Encoding: UTF-8
VignetteBuilder: knitr
RoxygenNote: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-08 12:53:35 UTC; meindl
Author: Bernhard Meindl [aut, cre]
Maintainer: Bernhard Meindl <bernhard.meindl@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-09 14:40:54 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-05-09 16:24:14 UTC; unix
Archs: sdcTable.so.dSYM
