# Generated by roxygen2: do not edit by hand

export(argusVersion)
export(attack)
export(calc.sdcProblem)
export(cell_info)
export(change_cellstatus)
export(contributing_indices)
export(createArgusInput)
export(createJJFormat)
export(createRegSDCInput)
export(get.dimInfo)
export(get.problemInstance)
export(get.sdcProblem)
export(getInfo)
export(makeProblem)
export(primarySuppression)
export(protectLinkedTables)
export(protectTable)
export(protect_linked_tables)
export(runArgusBatchFile)
export(sdcProb2df)
export(sdc_testproblem)
export(set.dimInfo)
export(set.problemInstance)
export(set.sdcProblem)
export(setInfo)
export(writeJJFormat)
exportClasses(cutList)
exportClasses(dataObj)
exportClasses(dimInfo)
exportClasses(dimVar)
exportClasses(linProb)
exportClasses(problemInstance)
exportClasses(safeObj)
exportClasses(sdcProblem)
exportClasses(simpleTriplet)
exportMethods(print)
exportMethods(show)
exportMethods(summary)
import(Rcpp)
import(data.table)
import(highs)
import(methods)
import(sdcHierarchies)
import(stringr)
importFrom(Matrix,sparseMatrix)
importFrom(SSBtools,GaussSuppression)
importFrom(knitr,knit)
importFrom(rlang,is_character)
importFrom(rlang,is_integerish)
importFrom(rlang,is_scalar_character)
importFrom(rlang,is_scalar_double)
importFrom(rlang,is_scalar_integerish)
importFrom(rlang,is_scalar_logical)
importFrom(slam,simple_triplet_matrix)
importFrom(stats,na.omit)
importFrom(utils,combn)
importFrom(utils,read.table)
importFrom(utils,str)
importFrom(utils,tail)
importFrom(utils,write.table)
useDynLib(sdcTable, .registration=TRUE)
