#' @useDynLib sdcTable, .registration=TRUE
#' @import methods
#' @import Rcpp
#' @import highs
#' @import stringr
#' @import data.table
#' @importFrom slam simple_triplet_matrix
#' @importFrom stats na.omit
#' @importFrom utils combn tail read.table str write.table
#' @import sdcHierarchies
#' @importFrom rlang is_scalar_character is_character is_integerish
#' @importFrom rlang is_scalar_double is_scalar_integerish is_scalar_logical
#' @importFrom knitr knit
#' @importFrom Matrix sparseMatrix
#' @importFrom SSBtools GaussSuppression
NULL
