#' Synthetic Microdata (1)
#'
#' A `data.frame` used for examples and problem-generation in various
#' examples.
#'
#' @format a `data.frame` with `100` rows and variables `region`, `gender`
#' and `val`.
#' @usage data(microdata1)
#' @examples
#' utils::data("microdata1", package = "sdcTable")
#' head(microdata1)
"microdata1"

#' Synthetic Microdata (2)
#'
#' Example microdata used for example in [protect_linked_tables()].
#'
#' @format a `data.frame` with `100` observations containing variables `region`,
#' `gender`,  `ecoOld`, `ecoNew` and `numVal`.
#' @usage data(microdata2)
#' @examples
#' utils::data("microdata2", package = "sdcTable")
#' head(microdata2)
"microdata2"

NULL
