# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

cpp_splitByIndices <- function(strings, strInfo) {
    .Call(`_sdcTable_cpp_splitByIndices`, strings, strInfo)
}

cpp_myPaste <- function(stringvec, nrKeyVars, seperator) {
    .Call(`_sdcTable_cpp_myPaste`, stringvec, nrKeyVars, seperator)
}

cpp_mySplit <- function(stringvec, indices) {
    .Call(`_sdcTable_cpp_mySplit`, stringvec, indices)
}

suppConstraints <- function(dat, m, params) {
    .Call(`_sdcTable_suppConstraints`, dat, m, params)
}

greedyMultDimSuppression <- function(dat, indices, subIndices, dimVars, verbose) {
    .Call(`_sdcTable_greedyMultDimSuppression`, dat, indices, subIndices, dimVars, verbose)
}

